'use strict';

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

exports.__esModule = true;

var _readFileRelative = require('read-file-relative');

var _utilsHttp = require('../utils/http');

// Const
var IDLE_PAGE_SCRIPT = _readFileRelative.readSync('../client/browser/idle-page/index.js');
var IDLE_PAGE_STYLE = _readFileRelative.readSync('../client/browser/idle-page/styles.css');
var IDLE_PAGE_LOGO = _readFileRelative.readSync('../client/browser/idle-page/logo.svg', true);

// Gateway

var BrowserConnectionGateway = (function () {
    function BrowserConnectionGateway(proxy) {
        _classCallCheck(this, BrowserConnectionGateway);

        this.connections = {};
        this.domain = proxy.server1Info.domain;

        this._registerRoutes(proxy);
    }

    BrowserConnectionGateway.prototype._dispatch = function _dispatch(url, proxy, handler) {
        var _this = this;

        proxy.GET(url, function (req, res, si, params) {
            var connection = _this.connections[params.id];

            _utilsHttp.preventCaching(res);

            if (connection) handler(req, res, connection);else _utilsHttp.respond404(res);
        });
    };

    BrowserConnectionGateway.prototype._registerRoutes = function _registerRoutes(proxy) {
        this._dispatch('/browser/connect/{id}', proxy, BrowserConnectionGateway.onConnection);
        this._dispatch('/browser/heartbeat/{id}', proxy, BrowserConnectionGateway.onHeartbeat);
        this._dispatch('/browser/idle/{id}', proxy, BrowserConnectionGateway.onIdle);
        this._dispatch('/browser/status/{id}', proxy, BrowserConnectionGateway.onStatusRequest);

        proxy.GET('/browser/assets/index.js', { content: IDLE_PAGE_SCRIPT, contentType: 'application/x-javascript' });
        proxy.GET('/browser/assets/styles.css', { content: IDLE_PAGE_STYLE, contentType: 'text/css' });
        proxy.GET('/browser/assets/logo.svg', { content: IDLE_PAGE_LOGO, contentType: 'image/svg+xml' });
    };

    // Helpers

    BrowserConnectionGateway.ensureConnectionReady = function ensureConnectionReady(res, connection) {
        if (!connection.ready) {
            _utilsHttp.respond500(res, 'The connection is not ready yet.');
            return false;
        }

        return true;
    };

    // Route handlers

    BrowserConnectionGateway.onConnection = function onConnection(req, res, connection) {
        if (connection.ready) _utilsHttp.respond500(res, 'The connection is already established.');else {
            var userAgent = req.headers['user-agent'];

            connection.establish(userAgent);
            _utilsHttp.redirect(res, connection.idleUrl);
        }
    };

    BrowserConnectionGateway.onHeartbeat = function onHeartbeat(req, res, connection) {
        if (BrowserConnectionGateway.ensureConnectionReady(res, connection)) {
            connection.heartbeat();
            res.end();
        }
    };

    BrowserConnectionGateway.onIdle = function onIdle(req, res, connection) {
        if (BrowserConnectionGateway.ensureConnectionReady(res, connection)) res.end(connection.renderIdlePage());
    };

    BrowserConnectionGateway.onStatusRequest = function onStatusRequest(req, res, connection) {
        if (BrowserConnectionGateway.ensureConnectionReady(res, connection)) {
            var status = connection.getStatus();

            _utilsHttp.respondWithJSON(res, status);
        }
    };

    // API

    BrowserConnectionGateway.prototype.startServingConnection = function startServingConnection(connection) {
        this.connections[connection.id] = connection;
    };

    BrowserConnectionGateway.prototype.stopServingConnection = function stopServingConnection(connection) {
        delete this.connections[connection.id];
    };

    return BrowserConnectionGateway;
})();

exports.default = BrowserConnectionGateway;
module.exports = exports.default;