'use strict';

var _inherits = require('babel-runtime/helpers/inherits').default;

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _events = require('events');

var _mustache = require('mustache');

var _mustache2 = _interopRequireDefault(_mustache);

var _useragent = require('useragent');

var _readFileRelative = require('read-file-relative');

var _command = require('./command');

var _command2 = _interopRequireDefault(_command);

var _messages = require('../messages');

var _utilsArrayRemove = require('../utils/array-remove');

var _utilsArrayRemove2 = _interopRequireDefault(_utilsArrayRemove);

// Const
var IDLE_PAGE_TEMPLATE = _readFileRelative.readSync('../client/browser/idle-page/index.html.mustache');

// Global instance counter used to generate ID's
var instanceCount = 0;

var BrowserConnection = (function (_EventEmitter) {
    _inherits(BrowserConnection, _EventEmitter);

    function BrowserConnection(gateway) {
        var _this = this;

        _classCallCheck(this, BrowserConnection);

        _EventEmitter.call(this);

        this.HEARTBEAT_TIMEOUT = 2 * 60 * 1000;

        this.id = ++instanceCount;
        this.jobQueue = [];
        this.browserConnectionGateway = gateway;
        this.userAgent = null;

        this.disconnected = false;
        this.ready = false;
        this.heartbeatTimeout = null;

        this.url = gateway.domain + '/browser/connect/' + this.id;
        this.heartbeatUrl = gateway.domain + '/browser/heartbeat/' + this.id;
        this.idleUrl = gateway.domain + '/browser/idle/' + this.id;
        this.statusUrl = gateway.domain + '/browser/status/' + this.id;

        this.on('error', function () {
            return _this.close();
        });

        this.browserConnectionGateway.startServingConnection(this);
    }

    BrowserConnection.prototype._waitForHeartbeat = function _waitForHeartbeat() {
        var _this2 = this;

        this.heartbeatTimeout = setTimeout(function () {
            _this2.emit('error', _messages.getText(_messages.MESSAGE.browserDisconnected, _this2.userAgent));
        }, this.HEARTBEAT_TIMEOUT);
    };

    BrowserConnection.prototype._popNextTestRunUrl = function _popNextTestRunUrl() {
        if (this.jobQueue.length) {
            var job = this.jobQueue[0];
            var url = job.popNextTestRunUrl();

            if (!job.hasQueuedTestRuns) this.jobQueue.shift();

            return url;
        }

        return null;
    };

    // API

    BrowserConnection.prototype.addJob = function addJob(job) {
        this.jobQueue.push(job);
    };

    BrowserConnection.prototype.removeJob = function removeJob(job) {
        _utilsArrayRemove2.default(this.jobQueue, job);
    };

    BrowserConnection.prototype.close = function close() {
        this.disconnected = true;
        this.ready = false;

        this.browserConnectionGateway.stopServingConnection(this);
        clearTimeout(this.heartbeatTimeout);
    };

    BrowserConnection.prototype.establish = function establish(userAgent) {
        this.ready = true;
        this.userAgent = _useragent.parse(userAgent).toString();

        this._waitForHeartbeat();
        this.emit('ready');
    };

    BrowserConnection.prototype.heartbeat = function heartbeat() {
        clearTimeout(this.heartbeatTimeout);
        this._waitForHeartbeat();
    };

    BrowserConnection.prototype.renderIdlePage = function renderIdlePage() {
        return _mustache2.default.render(IDLE_PAGE_TEMPLATE, {
            userAgent: this.userAgent,
            statusUrl: this.statusUrl,
            heartbeatUrl: this.heartbeatUrl
        });
    };

    BrowserConnection.prototype.getStatus = function getStatus() {
        var testRunUrl = this._popNextTestRunUrl();

        if (testRunUrl) return { cmd: _command2.default.run, url: testRunUrl };

        return { cmd: _command2.default.idle, url: this.idleUrl };
    };

    return BrowserConnection;
})(_events.EventEmitter);

exports.default = BrowserConnection;
module.exports = exports.default;