'use strict';

var _taggedTemplateLiteralLoose = require('babel-runtime/helpers/tagged-template-literal-loose').default;

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _Object$keys = require('babel-runtime/core-js/object/keys').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _templateObject = _taggedTemplateLiteralLoose(['\n    In the browser list, you can use aliases (e.g. "ie9", "chrome", etc.) as well as paths to executables.\n\n    To run tests against all installed browsers, use the "all" alias.\n\n    To use a remote browser connection (e.g. to connect a mobile device), specify "remote" as a browser alias. If you\n    need to connect multiple devices, prefix the alias with the number of browsers you want to connect (e.g., "3remote").\n\n    You can use one or more file paths or glob patterns to specify which tests to run.\n\n    More info: http://testcafe.devexpress.com/Documentation/CLI\n'], ['\n    In the browser list, you can use aliases (e.g. "ie9", "chrome", etc.) as well as paths to executables.\n\n    To run tests against all installed browsers, use the "all" alias.\n\n    To use a remote browser connection (e.g. to connect a mobile device), specify "remote" as a browser alias. If you\n    need to connect multiple devices, prefix the alias with the number of browsers you want to connect (e.g., "3remote").\n\n    You can use one or more file paths or glob patterns to specify which tests to run.\n\n    More info: http://testcafe.devexpress.com/Documentation/CLI\n']);

var _path = require('path');

var _commander = require('commander');

var _es6Promise = require('es6-promise');

var _es6Promisify = require('es6-promisify');

var _es6Promisify2 = _interopRequireDefault(_es6Promisify);

var _dedent = require('dedent');

var _dedent2 = _interopRequireDefault(_dedent);

var _globby = require('globby');

var _globby2 = _interopRequireDefault(_globby);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _testcafeBrowserNatives = require('testcafe-browser-natives');

var _readFileRelative = require('read-file-relative');

var _messages = require('../messages');

var _utilsGetViewportWidth = require('../utils/get-viewport-width');

var _utilsGetViewportWidth2 = _interopRequireDefault(_utilsGetViewportWidth);

var _utilsWordWrap = require('../utils/word-wrap');

var _utilsWordWrap2 = _interopRequireDefault(_utilsWordWrap);

var ensureDir = _es6Promisify2.default(_mkdirp2.default);

var REMOTE_ALIAS_RE = /^(\d*)remote$/;

var DESCRIPTION = _dedent2.default(_templateObject);

var CliArgumentParser = (function () {
    function CliArgumentParser() {
        _classCallCheck(this, CliArgumentParser);

        this.program = new _commander.Command('testcafe');

        this.cwd = process.cwd();

        this.src = null;
        this.browsers = null;
        this.filter = null;
        this.remoteCount = 0;
        this.opts = null;

        this._describeProgram();
    }

    CliArgumentParser._parsePortNumber = function _parsePortNumber(value) {
        var number = parseInt(value, 10);

        if (isNaN(value)) throw new Error(_messages.getText(_messages.MESSAGE.portNumberIsNotInteger));

        return number;
    };

    CliArgumentParser._optionValueToRegExp = function _optionValueToRegExp(name, value) {
        if (value === void 0) return value;

        try {
            return new RegExp(value);
        } catch (err) {
            throw new Error(_messages.getText(_messages.MESSAGE.optionValueIsNotValidRegExp, name));
        }
    };

    CliArgumentParser._replaceAllBrowsersAlias = function _replaceAllBrowsersAlias(browserList, browser, allAliases) {
        if (browser === 'all') browserList = browserList.concat(allAliases);else browserList.push(browser);

        return browserList;
    };

    CliArgumentParser._getDescription = function _getDescription() {
        // NOTE: add empty line to workaround commander-forced indentation on the first line.
        return '\n' + _utilsWordWrap2.default(DESCRIPTION, 2, _utilsGetViewportWidth2.default(process.stdout));
    };

    CliArgumentParser.prototype._describeProgram = function _describeProgram() {
        this.program.version(JSON.parse(_readFileRelative.readSync('../../package.json')).version).usage('[options] <comma-separated-browser-list> <file-or-glob ...>').description(CliArgumentParser._getDescription()).option('-b, --list-browsers', 'output the available browser aliases').option('-r, --reporter <name>', 'specify the reporter type to use').option('-s, --screenshots <path>', 'enable screenshot capturing and specify the path to save the screenshots to').option('-S, --screenshots-on-fails', 'take a screenshot whenever a test fails').option('-q, --quarantine-mode', 'enable the quarantine mode').option('-e, --skip-js-errors', 'make tests not fail when a JS error happens on a page').option('-t, --test <name>', 'run only tests with the specified name').option('-T, --test-grep <pattern>', 'run only tests matching the specified pattern').option('-f, --fixture <name>', 'run only fixtures with the specified name').option('-F, --fixture-grep <pattern>', 'run only fixtures matching the specified pattern').option('--ports <port1,port2>', 'specify custom port numbers').option('--hostname <name>', 'specify the hostname').option('--qr-code', 'outputs QR-code that repeats URLs used to connect the remote browsers')

        // NOTE: these options will be handled by chalk internally
        .option('--color', 'force colors in command line').option('--no-color', 'disable colors in command line');
    };

    CliArgumentParser.prototype._filterAndCountRemotes = function _filterAndCountRemotes(browser) {
        var remoteMatch = browser.match(REMOTE_ALIAS_RE);

        if (remoteMatch) {
            this.remoteCount += parseInt(remoteMatch[1], 10) || 1;
            return false;
        }

        return true;
    };

    CliArgumentParser.prototype._parseFilteringOptions = function _parseFilteringOptions() {
        var _this = this;

        this.opts.testGrep = CliArgumentParser._optionValueToRegExp('--test-grep', this.opts.testGrep);
        this.opts.fixtureGrep = CliArgumentParser._optionValueToRegExp('--fixture-grep', this.opts.fixtureGrep);

        this.filter = function (testName, fixtureName) {

            if (_this.opts.test && testName !== _this.opts.test) return false;

            if (_this.opts.testGrep && !_this.opts.testGrep.test(testName)) return false;

            if (_this.opts.fixture && fixtureName !== _this.opts.fixture) return false;

            if (_this.opts.fixtureGrep && !_this.opts.fixtureGrep.test(fixtureName)) return false;

            return true;
        };
    };

    CliArgumentParser.prototype._parsePorts = function _parsePorts() {
        return _regeneratorRuntime.async(function _parsePorts$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    if (!this.opts.ports) {
                        context$2$0.next = 4;
                        break;
                    }

                    this.opts.ports = this.opts.ports.split(',').map(CliArgumentParser._parsePortNumber);

                    if (!(this.opts.ports.length < 2)) {
                        context$2$0.next = 4;
                        break;
                    }

                    throw new Error(_messages.getText(_messages.MESSAGE.portsOptionRequiresTwoNumbers));

                case 4:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    CliArgumentParser.prototype._parseBrowserList = function _parseBrowserList() {
        var browsersArg, installations, allAliases;
        return _regeneratorRuntime.async(function _parseBrowserList$(context$2$0) {
            var _this2 = this;

            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    browsersArg = this.program.args[0] || '';
                    context$2$0.next = 3;
                    return _regeneratorRuntime.awrap(_testcafeBrowserNatives.getInstallations());

                case 3:
                    installations = context$2$0.sent;
                    allAliases = _Object$keys(installations);

                    this.browsers = browsersArg.split(',').filter(function (browser) {
                        return browser && _this2._filterAndCountRemotes(browser);
                    }).reduce(function (browserList, browser) {
                        return CliArgumentParser._replaceAllBrowsersAlias(browserList, browser, allAliases);
                    }, []);

                case 6:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    CliArgumentParser.prototype._parseFileList = function _parseFileList() {
        var fileList;
        return _regeneratorRuntime.async(function _parseFileList$(context$2$0) {
            var _this3 = this;

            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    fileList = this.program.args.slice(1);
                    context$2$0.next = 3;
                    return _regeneratorRuntime.awrap(_globby2.default(fileList, {
                        cwd: this.cwd,
                        silent: true,
                        nodir: true
                    }));

                case 3:
                    this.src = context$2$0.sent;

                    this.src = this.src.map(function (file) {
                        return _path.resolve(_this3.cwd, file);
                    });

                case 5:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    CliArgumentParser.prototype._parseScreenshotsPath = function _parseScreenshotsPath() {
        return _regeneratorRuntime.async(function _parseScreenshotsPath$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    if (!this.opts.screenshots) {
                        context$2$0.next = 4;
                        break;
                    }

                    this.opts.screenshots = _path.resolve(this.cwd, this.opts.screenshots);

                    context$2$0.next = 4;
                    return _regeneratorRuntime.awrap(ensureDir(this.opts.screenshots));

                case 4:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    CliArgumentParser.prototype.parse = function parse(argv) {
        return _regeneratorRuntime.async(function parse$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    this.program.parse(argv);

                    this.opts = this.program.opts();

                    // NOTE: the '-list-browsers' option only lists browsers and immediately exits the app.
                    // Therefore, we don't need to process other arguments.

                    if (this.opts.listBrowsers) {
                        context$2$0.next = 6;
                        break;
                    }

                    this._parseFilteringOptions();

                    context$2$0.next = 6;
                    return _regeneratorRuntime.awrap(_es6Promise.Promise.all([this._parsePorts(), this._parseScreenshotsPath(), this._parseBrowserList(), this._parseFileList()]));

                case 6:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    return CliArgumentParser;
})();

exports.default = CliArgumentParser;
module.exports = exports.default;