'use strict';

var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _Object$keys = require('babel-runtime/core-js/object/keys').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _testcafeBrowserNatives = require('testcafe-browser-natives');

var _argumentParser = require('./argument-parser');

var _argumentParser2 = _interopRequireDefault(_argumentParser);

var _log = require('./log');

var _log2 = _interopRequireDefault(_log);

var _remotesWizard = require('./remotes-wizard');

var _remotesWizard2 = _interopRequireDefault(_remotesWizard);

var _ = require('../');

var _2 = _interopRequireDefault(_);

function exit(code) {
    _log2.default.hideSpinner();

    // NOTE: give a process time to flush the output.
    // It's necessary in some environments.
    setTimeout(function () {
        return process.exit(code);
    }, 0);
}

function error(err) {
    _log2.default.hideSpinner();

    _log2.default.write(_chalk2.default.red('ERROR ') + err.message);
    _log2.default.write(_chalk2.default.gray('Type "tescafe -h" for help.'));

    exit(1);
}

function runTests(argParser) {
    var opts, port1, port2, testCafe, remoteBrowsers, browsers, runner, failed;
    return _regeneratorRuntime.async(function runTests$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                opts = argParser.opts;
                port1 = opts.ports && opts.ports[0];
                port2 = opts.ports && opts.ports[1];

                _log2.default.showSpinner();

                context$1$0.next = 6;
                return _regeneratorRuntime.awrap(_2.default(opts.hostname, port1, port2));

            case 6:
                testCafe = context$1$0.sent;
                context$1$0.next = 9;
                return _regeneratorRuntime.awrap(_remotesWizard2.default(testCafe, argParser.remoteCount, opts.qrCode));

            case 9:
                remoteBrowsers = context$1$0.sent;
                browsers = argParser.browsers.concat(remoteBrowsers);
                runner = testCafe.createRunner();

                runner.src(argParser.src).browsers(browsers).reporter(opts.reporter).filter(argParser.filter).screenshots(opts.screenshots, opts.screenshotsOnFails);

                runner.once('done-bootstrapping', function () {
                    return _log2.default.hideSpinner();
                });

                context$1$0.next = 16;
                return _regeneratorRuntime.awrap(runner.run({
                    skipJsErrors: opts.skipJsErrors,
                    quarantineMode: opts.quarantineMode
                }));

            case 16:
                failed = context$1$0.sent;

                exit(failed);

            case 18:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this);
}

function listBrowsers() {
    var installations, aliases;
    return _regeneratorRuntime.async(function listBrowsers$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                context$1$0.next = 2;
                return _regeneratorRuntime.awrap(_testcafeBrowserNatives.getInstallations());

            case 2:
                installations = context$1$0.sent;
                aliases = _Object$keys(installations);

                console.log(aliases.join('\n'));
                exit(0);

            case 6:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this);
}

(function cli() {
    var argParser;
    return _regeneratorRuntime.async(function cli$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                context$1$0.prev = 0;
                argParser = new _argumentParser2.default();
                context$1$0.next = 4;
                return _regeneratorRuntime.awrap(argParser.parse(process.argv));

            case 4:
                if (!argParser.opts.listBrowsers) {
                    context$1$0.next = 8;
                    break;
                }

                listBrowsers();
                context$1$0.next = 10;
                break;

            case 8:
                context$1$0.next = 10;
                return _regeneratorRuntime.awrap(runTests(argParser));

            case 10:
                context$1$0.next = 15;
                break;

            case 12:
                context$1$0.prev = 12;
                context$1$0.t0 = context$1$0['catch'](0);

                error(context$1$0.t0);

            case 15:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this, [[0, 12]]);
})();