'use strict';

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _Symbol = require('babel-runtime/core-js/symbol').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _indentString2 = require('indent-string');

var _indentString3 = _interopRequireDefault(_indentString2);

var _escapeHtml2 = require('escape-html');

var _escapeHtml3 = _interopRequireDefault(_escapeHtml2);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

require('moment-duration-format');

var _osFamily = require('os-family');

var _osFamily2 = _interopRequireDefault(_osFamily);

var _utilsWordWrap = require('../utils/word-wrap');

var _utilsWordWrap2 = _interopRequireDefault(_utilsWordWrap);

var _utilsGetViewportWidth = require('../utils/get-viewport-width');

var _utilsGetViewportWidth2 = _interopRequireDefault(_utilsGetViewportWidth);

var _errorsFormat = require('./errors/format');

var _errorsFormat2 = _interopRequireDefault(_errorsFormat);

var _mixinObject = require('mixin-object');

var _mixinObject2 = _interopRequireDefault(_mixinObject);

// NOTE: we should not expose internal state to
// the plugin, to avoid accidental rewrites.
// Therefore we use symbols to store them.
var stream = _Symbol();
var viewportWidth = _Symbol();
var wordWrapEnabled = _Symbol();
var indent = _Symbol();
var errorDecorator = _Symbol();

var ReporterPluginHost = (function () {
    function ReporterPluginHost(plugin, outStream) {
        _classCallCheck(this, ReporterPluginHost);

        this[stream] = outStream || process.stdout;
        this[viewportWidth] = _utilsGetViewportWidth2.default(this[outStream]);
        this[wordWrapEnabled] = false;
        this[indent] = 0;

        var useColors = this[stream] === process.stdout && _chalk2.default.enabled && !plugin.noColors;

        this.chalk = new _chalk2.default.constructor({ enabled: useColors });
        this.moment = _moment2.default;

        this.symbols = _osFamily2.default.win ? { ok: '√', err: '×' } : { ok: '✓', err: '✖' };

        _mixinObject2.default(this, plugin);

        this[errorDecorator] = this.createErrorDecorator();
    }

    // Error decorator

    ReporterPluginHost.prototype.createErrorDecorator = function createErrorDecorator() {
        var _this = this;

        return {
            'span category': function () {
                return '';
            },
            'span step-name': function (str) {
                return '"' + str + '"';
            },
            'span user-agent': function (str) {
                return _this.chalk.gray(str);
            },
            'div screenshot-info': function (str) {
                return str;
            },
            'a screenshot-path': function (str) {
                return _this.chalk.underline(str);
            },
            'code': function (str) {
                return _this.chalk.yellow(str);
            },
            'code step-source': function (str) {
                return _this.chalk.magenta(_this.indentString(str, 4));
            },
            'span code-line': function (str) {
                return str + '\n';
            },
            'span last-code-line': function (str) {
                return str;
            },
            'code api': function (str) {
                return _this.chalk.yellow(str);
            },
            'strong': function (str) {
                return _this.chalk.cyan(str);
            },
            'a': function (str) {
                return _this.chalk.yellow('"' + str + '"');
            }
        };
    };

    // String helpers

    ReporterPluginHost.prototype.indentString = function indentString(str, indentVal) {
        return _indentString3.default(str, ' ', indentVal);
    };

    ReporterPluginHost.prototype.wordWrap = function wordWrap(str, indentVal, width) {
        return _utilsWordWrap2.default(str, indentVal, width);
    };

    ReporterPluginHost.prototype.escapeHtml = function escapeHtml(str) {
        return _escapeHtml3.default(str);
    };

    ReporterPluginHost.prototype.formatError = function formatError(err) {
        var prefix = arguments.length <= 1 || arguments[1] === undefined ? '' : arguments[1];

        var maxMsgLength = this[viewportWidth] - this[indent] - prefix.length;
        var msg = _errorsFormat2.default(err, this[errorDecorator], maxMsgLength);

        msg = _utilsWordWrap2.default(msg, prefix.length, maxMsgLength);

        return prefix + msg.substr(prefix.length);
    };

    // Writing helpers

    ReporterPluginHost.prototype.newline = function newline() {
        this[stream].write('\n');

        return this;
    };

    ReporterPluginHost.prototype.write = function write(text) {
        if (this[wordWrapEnabled]) text = this.wordWrap(text, this[indent], this[viewportWidth]);else text = this.indentString(text, this[indent]);

        this[stream].write(text);

        return this;
    };

    ReporterPluginHost.prototype.useWordWrap = function useWordWrap(use) {
        this[wordWrapEnabled] = use;

        return this;
    };

    ReporterPluginHost.prototype.setIndent = function setIndent(val) {
        this[indent] = val;

        return this;
    };

    // Abstract methods implemented in plugin
    /* eslint-disable no-unused-vars */

    ReporterPluginHost.prototype.reportTaskStart = function reportTaskStart(startTime, userAgents) {
        throw new Error('Not implemented');
    };

    ReporterPluginHost.prototype.reportFixtureStart = function reportFixtureStart(name, path) {
        throw new Error('Not implemented');
    };

    ReporterPluginHost.prototype.reportTestDone = function reportTestDone(name, errs, durationMs, unstable, screenshotPath) {
        throw new Error('Not implemented');
    };

    ReporterPluginHost.prototype.reportTaskDone = function reportTaskDone(endTime, passed, total) {
        throw new Error('Not implemented');
    };

    /* eslint-enable no-unused-vars */
    return ReporterPluginHost;
})();

exports.default = ReporterPluginHost;
module.exports = exports.default;