'use strict';

exports.__esModule = true;
var NEW_LINE = '\n  ';

exports.default = function () {
    return {
        noColors: false,
        spaceLeft: 0,
        errDescriptors: [],
        currentFixtureName: null,

        reportTaskStart: function () {
            // NOTE: do nothing
        },

        reportFixtureStart: function (name) {
            this.currentFixtureName = name;
        },

        reportTestDone: function (name, errs) {
            var _this = this;

            var hasErr = !!errs.length;
            var dot = hasErr ? this.chalk.red('.') : '.';

            if (this.spaceLeft - 1 < 0) {
                this.spaceLeft = this.viewportWidth - NEW_LINE.length - 1;
                this.write(NEW_LINE);
            } else this.spaceLeft--;

            this.write(dot);

            if (hasErr) {
                this.errDescriptors = this.errDescriptors.concat(errs.map(function (err) {
                    return {
                        err: err,
                        testName: name,
                        fixtureName: _this.currentFixtureName
                    };
                }));
            }
        },

        reportTaskDone: function (endTime, passed, total) {
            var _this2 = this;

            var allPassed = !this.errDescriptors.length;
            var footer = allPassed ? this.chalk.bold.green(total + ' passed') : this.chalk.bold.red(total - passed + '/' + total + ' failed');

            this.setIndent(2).newline().newline().write(footer).newline();

            if (!allPassed) {
                this.errDescriptors.forEach(function (errDescriptor, idx) {
                    var prefix = idx + 1 + ') ';
                    var title = _this2.chalk.bold.red('' + prefix + errDescriptor.fixtureName + ' - ' + errDescriptor.testName);

                    _this2.setIndent(2).useWordWrap(true).newline().write(title).newline().setIndent(2 + prefix.length).write(_this2.formatError(errDescriptor.err)).newline().newline();
                });
            }
        }
    };
};

module.exports = exports.default;