'use strict';

exports.__esModule = true;

exports.default = function () {
    return {
        noColors: false,
        startTime: null,
        afterErrList: false,

        reportTaskStart: function (startTime, userAgents) {
            var _this = this;

            var uaList = userAgents.map(function (ua) {
                return _this.chalk.blue(ua);
            }).join(', ');

            this.startTime = startTime;

            this.setIndent(0).useWordWrap(true).write(this.chalk.bold('Running tests in: ' + uaList)).newline();
        },

        reportFixtureStart: function (name, path) {
            var title = name + ' (' + this.chalk.underline(path) + ')';

            this.setIndent(2).useWordWrap(true);

            if (this.afterErrList) this.afterErrList = false;else this.newline();

            this.write(title).newline();
        },

        reportTestDone: function (name, errs, durationMs, unstable, screenshotPath) {
            var _this2 = this;

            var hasErr = !!errs.length;
            var nameStyle = hasErr ? this.chalk.red : this.chalk.gray;
            var symbol = hasErr ? this.chalk.red(this.symbols.err) : this.chalk.green(this.symbols.ok);
            var title = symbol + ' ' + nameStyle(name);

            this.setIndent(4).useWordWrap(true);

            if (unstable) title += this.chalk.yellow(' (unstable)');

            if (screenshotPath) title += ' (screenshots: ' + this.chalk.underline(screenshotPath) + ')';

            this.write(title);

            if (hasErr) {
                this.setIndent(8).newline();

                errs.forEach(function (err, idx) {
                    _this2.newline().write(_this2.formatError(err, idx + 1 + ') ')).newline().newline();
                });
            }

            this.afterErrList = hasErr;

            this.newline();
        },

        reportTaskDone: function (endTime, passed, total) {
            var durationMs = endTime - this.startTime;
            var durationStr = this.moment.duration(durationMs).format('h[h] mm[m] ss[s]');
            var footer = passed === total ? this.chalk.bold.green(total + ' passed') : this.chalk.bold.red(total - passed + '/' + total + ' failed');

            footer += this.chalk.gray(' (' + durationStr + ')');

            this.setIndent(2).useWordWrap(true);

            if (!this.afterErrList) this.newline();

            this.newline().write(footer).newline();
        }
    };
};

module.exports = exports.default;