'use strict';

exports.__esModule = true;
var LINE_WIDTH = 100;

exports.default = function () {
    return {
        noColors: true,
        report: '',
        startTime: null,
        uaList: null,
        currentFixtureName: null,

        reportTaskStart: function (startTime, userAgents) {
            this.startTime = startTime;
            this.uaList = userAgents.join(', ');
        },

        reportFixtureStart: function (name) {
            this.currentFixtureName = this.escapeHtml(name);
        },

        reportTestDone: function (name, errs, durationMs, unstable, screenshotPath) {
            var _this = this;

            var hasErr = !!errs.length;

            if (unstable) name += ' (unstable)';

            if (screenshotPath) name += ' (screenshots: ' + screenshotPath + ')';

            name = this.escapeHtml(name);

            var openTag = '<testcase classname="' + this.currentFixtureName + '" name="' + name + '" time="' + durationMs / 1000 + '"';

            this.report += this.indentString(openTag, 2);

            if (hasErr) {
                this.report += ' >\n';
                this.report += this.indentString('<failure>\n', 4);
                this.report += this.indentString('<![CDATA[', 4);

                errs.forEach(function (err, idx) {
                    err = _this.escapeHtml(_this.formatError(err, idx + 1 + ') '));

                    _this.report += '\n';
                    _this.report += _this.wordWrap(err, 6, LINE_WIDTH);
                    _this.report += '\n';
                });

                this.report += this.indentString(']]>\n', 4);
                this.report += this.indentString('</failure>\n', 4);
                this.report += this.indentString('</testcase>\n', 2);
            } else this.report += ' />\n';
        },

        reportTaskDone: function (endTime, passed, total) {
            var name = 'TestCafe Tests: ' + this.escapeHtml(this.uaList);
            var failures = total - passed;
            var time = (endTime - this.startTime) / 1000;

            this.write('<?xml version="1.0" encoding="UTF-8" ?>').newline().write('<testsuite name="' + name + '" tests="' + total + '" failures="' + failures + '" ' + ('errors="' + failures + '" time="' + time + '" timestamp="' + endTime.toUTCString() + '" >')).newline().write(this.report).write('</testsuite>');
        }
    };
};

module.exports = exports.default;