'use strict';

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _es6Promise = require('es6-promise');

var _testcafeBrowserNatives = require('testcafe-browser-natives');

var _browserConnection = require('../browser-connection');

var _browserConnection2 = _interopRequireDefault(_browserConnection);

var _browserConnectionLocal = require('../browser-connection/local');

var _browserConnectionLocal2 = _interopRequireDefault(_browserConnectionLocal);

var _compiler = require('../compiler');

var _compiler2 = _interopRequireDefault(_compiler);

var _messages = require('../messages');

var Bootstrapper = (function () {
    function Bootstrapper(browserConnectionGateway) {
        _classCallCheck(this, Bootstrapper);

        this.BROWSER_CONNECTION_READY_TIMEOUT = 30 * 1000;

        this.browserConnectionGateway = browserConnectionGateway;

        this.sources = [];
        this.browsers = [];
        this.filter = null;
        this.reporter = null;
    }

    Bootstrapper._createBrowserConnectionsReadyPromise = function _createBrowserConnectionsReadyPromise(browserConnections) {
        var ready = browserConnections.filter(function (bc) {
            return !bc.ready;
        }).map(function (bc) {
            return new _es6Promise.Promise(function (resolve) {
                return bc.once('ready', resolve);
            });
        });

        return _es6Promise.Promise.all(ready);
    };

    Bootstrapper._convertAliasOrPathToBrowserInfo = function _convertAliasOrPathToBrowserInfo(browser) {
        var browserInfo;
        return _regeneratorRuntime.async(function _convertAliasOrPathToBrowserInfo$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    if (!(typeof browser === 'string')) {
                        context$2$0.next = 7;
                        break;
                    }

                    context$2$0.next = 3;
                    return _regeneratorRuntime.awrap(_testcafeBrowserNatives.getBrowserInfo(browser));

                case 3:
                    browserInfo = context$2$0.sent;

                    if (browserInfo) {
                        context$2$0.next = 6;
                        break;
                    }

                    throw new Error(_messages.getText(_messages.MESSAGE.cantFindBrowser, browser));

                case 6:
                    return context$2$0.abrupt('return', browserInfo);

                case 7:
                    return context$2$0.abrupt('return', browser);

                case 8:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    Bootstrapper.prototype._createConnectionFromBrowserInfo = function _createConnectionFromBrowserInfo(browserInfo) {
        if (browserInfo instanceof _browserConnection2.default) return browserInfo;

        return new _browserConnectionLocal2.default(this.browserConnectionGateway, browserInfo);
    };

    Bootstrapper.prototype._waitBrowserConnectionsReady = function _waitBrowserConnectionsReady(browserConnections) {
        var _this = this;

        return new _es6Promise.Promise(function (resolve, reject) {
            var timeout = setTimeout(function () {
                reject(new Error(_messages.getText(_messages.MESSAGE.cantEstablishBrowserConnection)));
            }, _this.BROWSER_CONNECTION_READY_TIMEOUT);

            var onError = function (msg) {
                clearTimeout(timeout);
                reject(new Error(msg));
            };

            browserConnections.forEach(function (bc) {
                return bc.once('error', onError);
            });

            Bootstrapper._createBrowserConnectionsReadyPromise(browserConnections).then(function () {
                browserConnections.forEach(function (bc) {
                    return bc.removeListener('error', onError);
                });
                clearTimeout(timeout);
                resolve();
            });
        });
    };

    Bootstrapper.prototype._checkForDisconnectedBrowsers = function _checkForDisconnectedBrowsers() {
        var disconnectedUserAgents = this.browsers.filter(function (browser) {
            return browser instanceof _browserConnection2.default && browser.disconnected;
        }).map(function (bc) {
            return bc.userAgent;
        });

        if (disconnectedUserAgents.length) throw new Error(_messages.getText(_messages.MESSAGE.cantRunAgainstDisconnectedBrowsers, disconnectedUserAgents.join(', ')));
    };

    Bootstrapper.prototype._getBrowserConnections = function _getBrowserConnections() {
        var browsers, browserConnections;
        return _regeneratorRuntime.async(function _getBrowserConnections$(context$2$0) {
            var _this2 = this;

            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    if (this.browsers.length) {
                        context$2$0.next = 2;
                        break;
                    }

                    throw new Error(_messages.getText(_messages.MESSAGE.browserNotSet));

                case 2:

                    this._checkForDisconnectedBrowsers();

                    context$2$0.next = 5;
                    return _regeneratorRuntime.awrap(_es6Promise.Promise.all(this.browsers.map(Bootstrapper._convertAliasOrPathToBrowserInfo)));

                case 5:
                    browsers = context$2$0.sent;
                    browserConnections = browsers.map(function (browser) {
                        return _this2._createConnectionFromBrowserInfo(browser);
                    });
                    context$2$0.prev = 7;
                    context$2$0.next = 10;
                    return _regeneratorRuntime.awrap(this._waitBrowserConnectionsReady(browserConnections));

                case 10:
                    context$2$0.next = 17;
                    break;

                case 12:
                    context$2$0.prev = 12;
                    context$2$0.t0 = context$2$0['catch'](7);
                    context$2$0.next = 16;
                    return _regeneratorRuntime.awrap(_es6Promise.Promise.all(browserConnections.map(function (bc) {
                        return new _es6Promise.Promise(function (resolve) {
                            if (bc instanceof _browserConnectionLocal2.default && !bc.disconnected) {
                                bc.close();
                                bc.once('closed', resolve);
                            } else resolve();
                        });
                    })));

                case 16:
                    throw context$2$0.t0;

                case 17:
                    return context$2$0.abrupt('return', browserConnections);

                case 18:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this, [[7, 12]]);
    };

    Bootstrapper.prototype._getTests = function _getTests() {
        var compiler, tests;
        return _regeneratorRuntime.async(function _getTests$(context$2$0) {
            var _this3 = this;

            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    if (this.sources.length) {
                        context$2$0.next = 2;
                        break;
                    }

                    throw new Error(_messages.getText(_messages.MESSAGE.testSourcesNotSet));

                case 2:
                    compiler = new _compiler2.default(this.sources);
                    context$2$0.next = 5;
                    return _regeneratorRuntime.awrap(compiler.getTests());

                case 5:
                    tests = context$2$0.sent;

                    if (this.filter) tests = tests.filter(function (test) {
                        return _this3.filter(test.name, test.fixture.name, test.fixture.path);
                    });

                    if (tests.length) {
                        context$2$0.next = 9;
                        break;
                    }

                    throw new Error(_messages.getText(_messages.MESSAGE.noTestsToRun));

                case 9:
                    return context$2$0.abrupt('return', tests);

                case 10:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    Bootstrapper.prototype._getReporterPlugin = function _getReporterPlugin() {
        var pluginFactory = this.reporter;

        if (typeof pluginFactory !== 'function') {
            try {
                // TODO
                var alias = this.reporter || 'spec';

                pluginFactory = require('../reporter/plugins/' + alias);
            } catch (err) {
                throw new Error(_messages.getText(_messages.MESSAGE.cantFindReporterForAlias, this.reporter));
            }
        }

        return pluginFactory();
    };

    // API

    Bootstrapper.prototype.createRunnableConfiguration = function createRunnableConfiguration() {
        var reporterPlugin, tests, browserConnections;
        return _regeneratorRuntime.async(function createRunnableConfiguration$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    reporterPlugin = this._getReporterPlugin();
                    context$2$0.next = 3;
                    return _regeneratorRuntime.awrap(this._getTests());

                case 3:
                    tests = context$2$0.sent;
                    context$2$0.next = 6;
                    return _regeneratorRuntime.awrap(this._getBrowserConnections());

                case 6:
                    browserConnections = context$2$0.sent;
                    return context$2$0.abrupt('return', { reporterPlugin: reporterPlugin, browserConnections: browserConnections, tests: tests });

                case 8:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    return Bootstrapper;
})();

exports.default = Bootstrapper;
module.exports = exports.default;