'use strict';

var _inherits = require('babel-runtime/helpers/inherits').default;

var _createClass = require('babel-runtime/helpers/create-class').default;

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _events = require('events');

var _testRun = require('./test-run');

var _testRun2 = _interopRequireDefault(_testRun);

// Const
var QUARANTINE_THRESHOLD = 3;

// Browser job

var BrowserJob = (function (_EventEmitter) {
    _inherits(BrowserJob, _EventEmitter);

    function BrowserJob(tests, browserConnection, proxy, screenshots, opts) {
        var _this = this;

        _classCallCheck(this, BrowserJob);

        _EventEmitter.call(this);

        this.started = false;
        this.quarantine = null;

        this.opts = opts;
        this.proxy = proxy;
        this.browserConnection = browserConnection;

        this.testRunQueue = tests.map(function (test) {
            return _this._createTestRun(test, screenshots);
        });
    }

    BrowserJob.prototype._shouldStartQuarantine = function _shouldStartQuarantine(testRun) {
        return !this.quarantine && testRun.errs.length;
    };

    BrowserJob.prototype._startQuarantine = function _startQuarantine(testRun) {
        this.quarantine = { passed: 0, failed: 1 };
        this._keepInQuarantine(testRun);
    };

    BrowserJob.prototype._endQuarantine = function _endQuarantine(testRun) {
        testRun.unstable = this.quarantine.passed > 0;
        this.quarantine = null;

        this._reportTestRunDone(testRun);
    };

    BrowserJob.prototype._shouldKeepInQuarantine = function _shouldKeepInQuarantine(testRun) {
        if (testRun.errs.length) this.quarantine.failed++;else this.quarantine.passed++;

        return this.quarantine.failed < QUARANTINE_THRESHOLD && this.quarantine.passed < QUARANTINE_THRESHOLD;
    };

    BrowserJob.prototype._keepInQuarantine = function _keepInQuarantine(testRun) {
        var nextAttempt = this._createTestRun(testRun.test, this.browserConnection);

        this.testRunQueue.splice(0, 0, nextAttempt);
    };

    BrowserJob.prototype._testRunDoneInQuarantineMode = function _testRunDoneInQuarantineMode(testRun) {
        this.proxy.closeSession(testRun);

        if (this._shouldStartQuarantine(testRun)) this._startQuarantine(testRun);else if (this.quarantine) {
            if (this._shouldKeepInQuarantine(testRun)) this._keepInQuarantine(testRun);else this._endQuarantine(testRun);
        } else this._reportTestRunDone(testRun);
    };

    BrowserJob.prototype._testRunDone = function _testRunDone(testRun) {
        this.proxy.closeSession(testRun);
        this._reportTestRunDone(testRun);
    };

    BrowserJob.prototype._reportTestRunDone = function _reportTestRunDone(testRun) {
        this.emit('test-run-done', testRun);

        if (!this.hasQueuedTestRuns) this.emit('done');
    };

    BrowserJob.prototype._createTestRun = function _createTestRun(test, screenshots) {
        var _this2 = this;

        var screenshotCapturer = screenshots.createCapturerFor(test, this.browserConnection.userAgent);
        var testRun = new _testRun2.default(test, this.browserConnection, screenshotCapturer, this.opts);
        var done = this.opts.quarantineMode ? function () {
            return _this2._testRunDoneInQuarantineMode(testRun);
        } : function () {
            return _this2._testRunDone(testRun);
        };

        testRun.once('start', function () {
            return _this2.emit('test-run-start', testRun);
        });
        testRun.once('done', done);

        return testRun;
    };

    // API

    BrowserJob.prototype.popNextTestRunUrl = function popNextTestRunUrl() {
        var testRun = this.testRunQueue.shift();

        if (!this.started) {
            this.started = true;
            this.emit('start');
        }

        if (testRun) return this.proxy.openSession(testRun.test.fixture.page, testRun);

        return null;
    };

    _createClass(BrowserJob, [{
        key: 'hasQueuedTestRuns',
        get: function () {
            return !!this.testRunQueue.length;
        }
    }]);

    return BrowserJob;
})(_events.EventEmitter);

exports.default = BrowserJob;
module.exports = exports.default;