'use strict';

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _path = require('path');

var _uuid = require('uuid');

var _uuid2 = _interopRequireDefault(_uuid);

var _arrayFind = require('array-find');

var _arrayFind2 = _interopRequireDefault(_arrayFind);

var _capturer = require('./capturer');

var _capturer2 = _interopRequireDefault(_capturer);

var Screenshots = (function () {
    function Screenshots(path) {
        _classCallCheck(this, Screenshots);

        this.enabled = !!path;
        this.path = path;
        this.testEntries = [];
    }

    Screenshots._escapeUserAgent = function _escapeUserAgent(userAgent) {
        return userAgent.toString().split('/').map(function (str) {
            return str.trim().replace(/\s/g, '_');
        }).join('_');
    };

    Screenshots.prototype._addTestEntry = function _addTestEntry(test) {
        var testEntry = {
            test: test,
            path: this.path ? _path.join(this.path, _uuid2.default.v4().substr(0, 8)) : '',
            hasScreenshots: false
        };

        this.testEntries.push(testEntry);

        return testEntry;
    };

    Screenshots.prototype._getTestEntry = function _getTestEntry(test) {
        return _arrayFind2.default(this.testEntries, function (entry) {
            return entry.test === test;
        });
    };

    Screenshots.prototype.hasCapturedFor = function hasCapturedFor(test) {
        return this._getTestEntry(test).hasScreenshots;
    };

    Screenshots.prototype.getPathFor = function getPathFor(test) {
        return this._getTestEntry(test).path;
    };

    Screenshots.prototype.createCapturerFor = function createCapturerFor(test, userAgent) {
        var testEntry = this._getTestEntry(test);

        if (!testEntry) testEntry = this._addTestEntry(test);

        var screenshotPath = this.enabled ? _path.join(testEntry.path, Screenshots._escapeUserAgent(userAgent)) : null;

        return new _capturer2.default(screenshotPath, _path.dirname(test.fixture.path), testEntry);
    };

    return Screenshots;
})();

exports.default = Screenshots;
module.exports = exports.default;