'use strict';

var _inherits = require('babel-runtime/helpers/inherits').default;

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _events = require('events');

var _browserJob = require('./browser-job');

var _browserJob2 = _interopRequireDefault(_browserJob);

var _screenshots = require('./screenshots');

var _screenshots2 = _interopRequireDefault(_screenshots);

var _utilsArrayRemove = require('../utils/array-remove');

var _utilsArrayRemove2 = _interopRequireDefault(_utilsArrayRemove);

var Task = (function (_EventEmitter) {
    _inherits(Task, _EventEmitter);

    function Task(tests, browserConnections, proxy, opts) {
        _classCallCheck(this, Task);

        _EventEmitter.call(this);

        this.running = false;
        this.browserConnections = browserConnections;
        this.tests = tests;
        this.screenshots = new _screenshots2.default(opts.screenshotPath);

        this.pendingBrowserJobs = this._createBrowserJobs(tests, proxy, this.screenshots, opts);
    }

    Task.prototype._assignBrowserJobEventHandlers = function _assignBrowserJobEventHandlers(job) {
        var _this = this;

        job.on('test-run-start', function (testRun) {
            return _this.emit('test-run-start', testRun);
        });
        job.on('test-run-done', function (testRun) {
            return _this.emit('test-run-done', testRun);
        });

        job.once('start', function () {
            if (!_this.running) {
                _this.running = true;
                _this.emit('start');
            }
        });

        job.once('done', function () {
            _utilsArrayRemove2.default(_this.pendingBrowserJobs, job);
            _this.emit('browser-job-done', job);

            if (!_this.pendingBrowserJobs.length) _this.emit('done');
        });
    };

    Task.prototype._createBrowserJobs = function _createBrowserJobs(tests, proxy, screenshots, opts) {
        var _this2 = this;

        return this.browserConnections.map(function (bc) {
            var job = new _browserJob2.default(tests, bc, proxy, screenshots, opts);

            _this2._assignBrowserJobEventHandlers(job);
            bc.addJob(job);

            return job;
        });
    };

    // API

    Task.prototype.abort = function abort() {
        this.pendingBrowserJobs.forEach(function (job) {
            job.removeAllListeners();
            job.browserConnection.removeJob(job);
        });
    };

    return Task;
})(_events.EventEmitter);

exports.default = Task;
module.exports = exports.default;