'use strict';

var _inherits = require('babel-runtime/helpers/inherits').default;

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _readFileRelative = require('read-file-relative');

var _mustache = require('mustache');

var _mustache2 = _interopRequireDefault(_mustache);

var _testcafeHammerhead = require('testcafe-hammerhead');

var _command = require('./command');

var _command2 = _interopRequireDefault(_command);

var _reporterErrorsType = require('../../reporter/errors/type');

var _reporterErrorsType2 = _interopRequireDefault(_reporterErrorsType);

// Const
var TEST_RUN_TEMPLATE = _readFileRelative.readSync('../../client/test-run/index.js.mustache');
var IFRAME_TEST_RUN_TEMPLATE = _readFileRelative.readSync('../../client/test-run/iframe.js.mustache');

var TestRun = (function (_Session) {
    _inherits(TestRun, _Session);

    function TestRun(test, browserConnection, screenshotCapturer, opts) {
        _classCallCheck(this, TestRun);

        var uploadsRoot = _path2.default.dirname(test.fixture.path);

        _Session.call(this, uploadsRoot);

        this.running = false;
        this.unstable = false;

        this.opts = opts;
        this.test = test;
        this.browserConnection = browserConnection;

        this.isFileDownloading = false;

        // TODO remove it then we move shared data to session storage
        this.errs = [];
        this.deferredError = null;
        this.restartCount = 0;
        this.nextStep = 0;
        this.actionTargetWaiting = 0;
        this.nativeDialogsInfo = null;
        this.nativeDialogsInfoTimeStamp = 0;
        this.stepsSharedData = {};
        this.screenshotCapturer = screenshotCapturer;

        this.injectable.scripts.push('/testcafe-core.js');
        this.injectable.scripts.push('/testcafe-ui.js');
        this.injectable.scripts.push('/testcafe-runner.js');
        this.injectable.styles.push('/testcafe-ui-styles.css');
    }

    // Service message handlers

    TestRun.prototype._getPayloadScript = function _getPayloadScript() {
        var sharedJs = this.test.fixture.getSharedJs();

        // TODO
        var nextStep = this.actionTargetWaiting ? this.nextStep - 1 : this.nextStep;

        this.actionTargetWaiting = false;

        if (!this.running) {
            this.running = true;
            this.emit('start');
        }

        return _mustache2.default.render(TEST_RUN_TEMPLATE, {
            stepNames: JSON.stringify(this.test.stepData.names),
            testSteps: this.test.stepData.js,
            sharedJs: sharedJs,
            nextStep: nextStep,
            deferredError: this.deferredError ? JSON.stringify(this.deferredError) : 'null',
            browserHeartbeatUrl: this.browserConnection.heartbeatUrl,
            browserStatusUrl: this.browserConnection.statusUrl,
            takeScreenshots: this.screenshotCapturer.enabled,
            takeScreenshotsOnFails: this.opts.takeScreenshotsOnFails,
            skipJsErrors: this.opts.skipJsErrors,
            nativeDialogsInfo: JSON.stringify(this.nativeDialogsInfo),
            iFrameTestRunScript: JSON.stringify(this._getIframePayloadScript())
        });
    };

    TestRun.prototype._getIframePayloadScript = function _getIframePayloadScript() {
        var sharedJs = this.test.fixture.getSharedJs();

        return _mustache2.default.render(IFRAME_TEST_RUN_TEMPLATE, {
            sharedJs: sharedJs,
            takeScreenshotsOnFails: this.opts.takeScreenshotsOnFails,
            skipJsErrors: this.opts.skipJsErrors,
            nativeDialogsInfo: JSON.stringify(this.nativeDialogsInfo)
        });
    };

    TestRun.prototype._addError = function _addError(err) {
        return _regeneratorRuntime.async(function _addError$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    if (err.__sourceIndex !== void 0 && err.__sourceIndex !== null) {
                        err.relatedSourceCode = this.test.sourceIndex[err.__sourceIndex];
                        delete err.__sourceIndex;
                    }

                    context$2$0.prev = 1;
                    context$2$0.next = 4;
                    return _regeneratorRuntime.awrap(this.screenshotCapturer.captureError(err));

                case 4:
                    err.screenshotPath = context$2$0.sent;
                    context$2$0.next = 9;
                    break;

                case 7:
                    context$2$0.prev = 7;
                    context$2$0.t0 = context$2$0['catch'](1);

                case 9:
                    // NOTE: swallow the error silently if we can't take screenshots for some
                    // reason (e.g. we don't have permissions to write a screenshot file).

                    this.errs.push(err);

                case 10:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this, [[1, 7]]);
    };

    TestRun.prototype._fatalError = function _fatalError(err, deferred) {
        return _regeneratorRuntime.async(function _fatalError$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    if (!deferred) {
                        context$2$0.next = 3;
                        break;
                    }

                    this.deferredError = err;
                    return context$2$0.abrupt('return');

                case 3:
                    context$2$0.next = 5;
                    return _regeneratorRuntime.awrap(this._addError(err));

                case 5:
                    this.emit('done');

                case 6:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    TestRun.prototype.getAuthCredentials = function getAuthCredentials() {
        return this.test.fixture.authCredentials;
    };

    TestRun.prototype.handleFileDownload = function handleFileDownload() {
        this.isFileDownloading = true;
    };

    TestRun.prototype.handlePageError = function handlePageError(ctx, errMsg) {
        this._fatalError({
            type: _reporterErrorsType2.default.pageNotLoaded,
            message: errMsg
        });

        ctx.redirect(this.browserConnection.idleUrl);
    };

    return TestRun;
})(_testcafeHammerhead.Session);

exports.default = TestRun;
var ServiceMessages = TestRun.prototype;

ServiceMessages[_command2.default.fatalError] = function (msg) {
    return this._fatalError(msg.err, msg.deferred);
};

ServiceMessages[_command2.default.assertionFailed] = function (msg) {
    return this._addError(msg.err);
};

ServiceMessages[_command2.default.done] = function () {
    this.emit('done');
};

ServiceMessages[_command2.default.setStepsSharedData] = function (msg) {
    this.stepsSharedData = msg.stepsSharedData;
};

ServiceMessages[_command2.default.getStepsSharedData] = function () {
    return this.stepsSharedData;
};

ServiceMessages[_command2.default.setNextStep] = function (msg) {
    this.nextStep = msg.nextStep;
};

ServiceMessages[_command2.default.setActionTargetWaiting] = function (msg) {
    this.actionTargetWaiting = msg.value;
};

ServiceMessages[_command2.default.getAndUncheckFileDownloadingFlag] = function () {
    var isFileDownloading = this.isFileDownloading;

    this.isFileDownloading = false;

    return isFileDownloading;
};

ServiceMessages[_command2.default.uncheckFileDownloadingFlag] = function () {
    this.isFileDownloading = false;
};

ServiceMessages[_command2.default.nativeDialogsInfoSet] = function (msg) {
    if (msg.timeStamp >= this.nativeDialogsInfoTimeStamp) {
        //NOTE: the server can get messages in the wrong sequence if they was sent with a little distance (several milliseconds),
        // we don't take to account old messages
        this.nativeDialogsInfoTimeStamp = msg.timeStamp;
        this.nativeDialogsInfo = msg.info;
    }
};

ServiceMessages[_command2.default.takeScreenshot] = function callee$0$0(msg) {
    return _regeneratorRuntime.async(function callee$0$0$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                context$1$0.prev = 0;
                context$1$0.next = 3;
                return _regeneratorRuntime.awrap(this.screenshotCapturer.captureAction(msg));

            case 3:
                return context$1$0.abrupt('return', context$1$0.sent);

            case 6:
                context$1$0.prev = 6;
                context$1$0.t0 = context$1$0['catch'](0);
                return context$1$0.abrupt('return', null);

            case 9:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this, [[0, 6]]);
};
module.exports = exports.default;

// TODO: move this logic to the client when localStorageSandbox will be implemented in the
// testcafe-hammerhead repo https://github.com/DevExpress/testcafe-hammerhead/issues/252 !!!

// NOTE: swallow the error silently if we can't take screenshots for some
// reason (e.g. we don't have permissions to write a screenshot file).