'use strict';

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _readFileRelative = require('read-file-relative');

var _testcafeHammerhead = require('testcafe-hammerhead');

var _browserConnectionGateway = require('./browser-connection/gateway');

var _browserConnectionGateway2 = _interopRequireDefault(_browserConnectionGateway);

var _browserConnection = require('./browser-connection');

var _browserConnection2 = _interopRequireDefault(_browserConnection);

var _runner = require('./runner');

var _runner2 = _interopRequireDefault(_runner);

// Const
var CORE_SCRIPT = _readFileRelative.readSync('./client/core/index.js');
var RUNNER_SCRIPT = _readFileRelative.readSync('./client/runner/index.js');
var UI_SCRIPT = _readFileRelative.readSync('./client/ui/index.js');
var UI_STYLE = _readFileRelative.readSync('./client/ui/styles.css');
var UI_SPRITE = _readFileRelative.readSync('./client/ui/sprite.png', true);
var FAVICON = _readFileRelative.readSync('./client/ui/favicon.ico', true);

var TestCafe = (function () {
    function TestCafe(hostname, port1, port2) {
        _classCallCheck(this, TestCafe);

        this.proxy = new _testcafeHammerhead.Proxy(hostname, port1, port2);
        this.browserConnectionGateway = new _browserConnectionGateway2.default(this.proxy);

        this._registerAssets();
    }

    TestCafe.prototype._registerAssets = function _registerAssets() {
        this.proxy.GET('/testcafe-core.js', { content: CORE_SCRIPT, contentType: 'application/x-javascript' });
        this.proxy.GET('/testcafe-runner.js', { content: RUNNER_SCRIPT, contentType: 'application/x-javascript' });
        this.proxy.GET('/testcafe-ui.js', { content: UI_SCRIPT, contentType: 'application/x-javascript' });
        this.proxy.GET('/testcafe-ui-sprite.png', { content: UI_SPRITE, contentType: 'image/png' });
        this.proxy.GET('/favicon.ico', { content: FAVICON, contentType: 'image/x-icon' });

        this.proxy.GET('/testcafe-ui-styles.css', {
            content: UI_STYLE,
            contentType: 'text/css',
            isShadowUIStylesheet: true
        });
    };

    // API

    TestCafe.prototype.createBrowserConnection = function createBrowserConnection() {
        return new _browserConnection2.default(this.browserConnectionGateway);
    };

    TestCafe.prototype.createRunner = function createRunner() {
        return new _runner2.default(this.proxy, this.browserConnectionGateway);
    };

    TestCafe.prototype.close = function close() {
        this.proxy.close();
    };

    return TestCafe;
})();

exports.default = TestCafe;
module.exports = exports.default;